%
%
%  Tracks Split Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Tracks Functions">
%        <Item name="Split Tracks" icon="Matlab">
%          <Command>MatlabXT::XTTracksSplit(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Split Tracks">
%            <Command>MatlabXT::XTTracksSplit(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Split Tracks">
%            <Command>MatlabXT::XTTracksSplit(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%   
%   Create a new container with splitted tracks. 
%   
%

function XTTracksSplit(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% the user has to create a scene with some surpass components
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  msgbox('Please create some tracks in the surpass scene!');
  return
end

% get the selected object (spots or surfaces)
% read number of objects
vFactory = vImarisApplication.GetFactory;
vObjects = vImarisApplication.GetSurpassSelection;
if ~vFactory.IsSpots(vObjects) && ~vFactory.IsSurfaces(vObjects)
  for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
    vObjects = vSurpassScene.GetChild(vChildIndex - 1);
    if vFactory.IsSpots(vObjects) || vFactory.IsSurfaces(vObjects)
      break
    end
  end
end
if vFactory.IsSpots(vObjects)
  vObjects = vFactory.ToSpots(vObjects);
  vCoords = vObjects.GetPositionsXYZ;
  vTimes = vObjects.GetIndicesT;
  vRadii = vObjects.GetRadiiXYZ;
  vNumberOfObjects = numel(vTimes);
elseif vFactory.IsSurfaces(vObjects)
  vObjects = vFactory.ToSurfaces(vObjects);
  vNumberOfObjects = vObjects.GetNumberOfSurfaces;
else
  msgbox('Please select some spots or surfaces!')
  return
end


% get the track edges (same code for spots and surfaces)
vEdges = vObjects.GetTrackEdges + 1; % indices start from 1 here (matlab)
if isempty(vEdges)  
  msgbox('Please select some tracks!')
  return
end
vNumberOfEdges = size(vEdges, 1);

vProgressDisplay = waitbar(0, 'Analyzing Tracks');

% assign every object and edge to a track index
vTrackIndexObject = zeros(vNumberOfObjects, 1);
vTrackIndexEdge = zeros(vNumberOfEdges, 1);
vTracksCount = 0;
for vIndex = 1:vNumberOfEdges
  % use some shorter names...
  vE1 = vEdges(vIndex, 1);
  vE2 = vEdges(vIndex, 2);
  vI1 = vTrackIndexObject(vE1);
  vI2 = vTrackIndexObject(vE2);
  
  if vI1 == 0 && vI2 == 0
    % two free objects found -> add new track
    vTracksCount = vTracksCount + 1;
    vTrackIndexObject([vE1, vE2]) = vTracksCount;
    vTrackIndexEdge(vIndex) = vTracksCount;
  elseif vI1 ~= 0 && vI2 == 0
    % first object is free -> add it to the track of the second object
    vTrackIndexObject(vE2) = vI1;
    vTrackIndexEdge(vIndex) = vI1;
  elseif vI1 == 0 && vI2 ~= 0
    % second object is free -> add it to the track of the first object
    vTrackIndexObject(vE1) = vI2;
    vTrackIndexEdge(vIndex) = vI2;
  elseif vI1 ~= vI2
    % two objects belongs to different partial tracks -> merge
    vTrackIndexObject(vTrackIndexObject == vI2) = vI1;
    vTrackIndexEdge(vTrackIndexEdge == vI2) = vI1;
  end
  
  waitbar(0.5*vIndex/vNumberOfEdges, vProgressDisplay)
end

waitbar(0.5, vProgressDisplay, 'Creating Tracks')
vProgress = 0;

% create a container for the new objects
vParent = vFactory.CreateDataContainer;
vParent.SetName(char(vObjects.GetName));

% build the surpass components
vIndices = 1:vNumberOfObjects;
vTrackIndex = 0; % don't count empty indices (created by merge operations)
for vIndex = [1:vTracksCount, 0]
  vObjectIndices = vIndices(vTrackIndexObject == vIndex);
  if isempty(vObjectIndices)
    continue
  end
  
  % objects
  if vFactory.IsSpots(vObjects)
    vNewObjects = vFactory.CreateSpots;
    vNewObjects.Set(vCoords(vObjectIndices, :), vTimes(vObjectIndices), ...
	  zeros(sum(vObjectIndices~=0),1));
	vNewObjects.SetRadiiXYZ(vRadii(vObjectIndices,:));
  else
    vNewObjects = vFactory.CreateSurfaces;
    for vSurface = vObjectIndices - 1
      vVertices = vObjects.GetVertices(vSurface);
      vTriangles = vObjects.GetTriangles(vSurface);
      vNormals = vObjects.GetNormals(vSurface);
      vTime = vObjects.GetTimeIndex(vSurface);
      vNewObjects.AddSurface(vVertices, vTriangles, vNormals, vTime);
    end
  end
  
  % edges
  if vIndex == 0 % free objects
    vNewObjects.SetName(sprintf('%s Free Objects', char(vObjects.GetName)));
  else
    vEdgeIndices = vEdges(vTrackIndexEdge == vIndex, :);
    vMap = zeros(vNumberOfObjects, 1);
    vMap(vObjectIndices) = 1:numel(vObjectIndices);
    vEdgeIndices = vMap(vEdgeIndices);
    if size(vEdgeIndices, 2) == 1
      vEdgeIndices = vEdgeIndices';
    end
    
    vNewObjects.SetTrackEdges(vEdgeIndices - 1);
    vTrackIndex = vTrackIndex + 1;
    vNewObjects.SetName(sprintf('%s Track %i', char(vObjects.GetName), vTrackIndex));
  end
  
  % colorize and add to the container
  vNewObjects.SetColorRGBA(floor((256^3)*rand));
  vParent.AddChild(vNewObjects, -1);

  vProgress = vProgress + numel(vObjectIndices);
  waitbar(0.5 + 0.5*vProgress/vNumberOfObjects, vProgressDisplay)
end
close(vProgressDisplay)

% add to the surpass scene
vObjects.GetParent.AddChild(vParent, -1);
vObjects.SetVisible(0);
